searchData={"items":[{"type":"task","title":"mix yacto","doc":"Prints Yacto tasks and their information.\n\n    mix yacto","ref":"Mix.Tasks.Yacto.html"},{"type":"task","title":"mix yacto.gen.migration","doc":"","ref":"Mix.Tasks.Yacto.Gen.Migration.html"},{"type":"function","title":"Mix.Tasks.Yacto.Gen.Migration.run/1","doc":"","ref":"Mix.Tasks.Yacto.Gen.Migration.html#run/1"},{"type":"task","title":"mix yacto.migrate","doc":"","ref":"Mix.Tasks.Yacto.Migrate.html"},{"type":"function","title":"Mix.Tasks.Yacto.Migrate.run/1","doc":"","ref":"Mix.Tasks.Yacto.Migrate.html#run/1"},{"type":"module","title":"Yacto","doc":"Yacto は、[Ecto](https://hexdocs.pm/ecto/Ecto.html) で手が届きにくい部分をサポートするためのライブラリです。\n\n大まかに以下の機能があります。\n\n- マイグレーションファイルの自動生成\n- 別アプリケーションからのマイグレーションの利用\n- 水平分割したデータベースへのマイグレーション\n- 複数データベースを跨るトランザクション（XA トランザクション）","ref":"Yacto.html"},{"type":"module","title":"マイグレーションファイルの自動生成 - Yacto","doc":"Yacto は、特にマイグレーション周りが Ecto と異なります。\nEcto はスキーマとマイグレーションを別で定義していましたが、Yacto は **スキーマからマイグレーションファイルを自動的に出力します**。\n\n具体的には、以下の様にスキーマを定義したとして、\n\nlib/my_app/player.ex:\n\n```elixir\ndefmodule MyApp.Player do\n  use Yacto.Schema, dbname: :player\n\n  schema @auto_source do\n    # sharding key\n    field :player_id, :string, meta: [null: false, size: 64]\n    field :hp, :integer, default: 0, meta: [null: false]\n    index :player_id, unique: true\n  end\nend\n```\n\nこの状態で `mix yacto.gen.migration` を実行すると、以下の様なマイグレーションファイルが出力されます。\n\n`priv/migrations/Elixir.MyApp.Player/0000-player-create-2020_04_22_085825.exs`:\n\n```elixir\ndefmodule MyApp.Player.Migration0000 do\n  use Ecto.Migration\n\n  def change() do\n    create table(\"myapp_player\")\n    alter table(\"myapp_player\") do\n      add(:hp, :integer, [null: false, size: 64])\n      add(:player_id, :string, [null: false])\n    end\n    create index(\"myapp_player\", [:player_id], [name: \"player_id_index\", unique: true])\n  end\n\n  def __migration__(:structure) do\n    {MyApp.Player, %Yacto.Migration.Structure{fields: [:id, :player_id, :hp], meta: %{attrs: %{hp: %{null: false}, player_id: %{null: false, size: 64}}, indices: %{{[:player_id], [unique: true]} => true}}, source: \"myapp_player\", types: %{hp: :integer, id: :id, player_id: :string}}},\n  end\n\n  def __migration__(:version) do\n    0\n  end\nend\n```\n\nあとは `mix yacto.migrate` を実行すれば、このマイグレーションファイルがデータベースに反映されます。\nもうマイグレーションファイルを自分で記述する必要はありません。\n\n更に、この状態で `MyApp.Player` スキーマに `:mp` フィールドを追加して `mix yacto.gen.migration` を実行すると、以下のマイグレーションファイルが生成されます。\n\nlib/my_app/player.ex:\n\n```elixir\ndefmodule MyApp.Player do\n  ...\n\n  schema @auto_source do\n    ...\n\n    # add a field\n    field :mp, :integer, default: 0, meta: [null: false]\n\n    ...\n  end\nend\n```\n\n`priv/migrations/Elixir.MyApp.Player/0001-player-change-2020_04_22_092258.exs`:\n\n```elixir\ndefmodule MyApp.Player.Migration0001 do\n  use Ecto.Migration\n\n  def change() do\n    alter table(\"myapp_player\") do\n      add(:mp, :integer, [null: false])\n    end\n  end\n\n  def __migration__(:structure) do\n    [\n      {MyApp.Player, %Yacto.Migration.Structure{fields: [:id, :player_id, :hp, :mp], meta: %{attrs: %{hp: %{null: false}, mp: %{null: false}, player_id: %{null: false, size: 64}}, indices: %{{[:player_id], [unique: true]} => true}}, source: \"myapp_player\", types: %{hp: :integer, id: :id, mp: :integer, player_id: :string}}},\n    ]\n  end\n\n  def __migration__(:version) do\n    1\n  end\nend\n```\n\nこのように、以前からの差分だけがマイグレーションファイルに出力されます。\n`mix yacto.migrate` を実行すれば、このマイグレーションファイルがデータベースに反映されます。\n\nもしマイグレーションファイルが１つもデータベースに適用されていなかったら、上記の２つのマイグレーションファイルが順番に適用されます。\n\n#","ref":"Yacto.html#module-%E3%83%9E%E3%82%A4%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%83%95%E3%82%A1%E3%82%A4%E3%83%AB%E3%81%AE%E8%87%AA%E5%8B%95%E7%94%9F%E6%88%90"},{"type":"module","title":"別アプリケーションからのマイグレーションの利用 - Yacto","doc":"先程作った `my_app` アプリケーションを利用する `other_app` アプリケーションがあったとします。\n`my_app` はデータベース利用しているので、`other_app` 上で `my_app` のためのマイグレーションを行う必要があります。\nYacto を使えば、`config/config.exs` を適切に書いた後、`other_app` で以下のコマンドを実行するだけで `my_app` のマイグレーションができます。\n\n```\nmix yacto.migrate --app my_app\n```\n\nEcto では、他のアプリケーションが必要としているマイグレーションを自分で書くか、各アプリケーションが指定したバラバラな方法でマイグレーションを行う必要がありました。\nYacto を使っているアプリケーションでは、全て同じ方法でマイグレーションができます。\n\n#","ref":"Yacto.html#module-%E5%88%A5%E3%82%A2%E3%83%97%E3%83%AA%E3%82%B1%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%81%8B%E3%82%89%E3%81%AE%E3%83%9E%E3%82%A4%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%81%AE%E5%88%A9%E7%94%A8"},{"type":"module","title":"水平分割したデータベースへのマイグレーション - Yacto","doc":"例えば `MyApp.Player` スキーマを水平分割した場合、このスキーマのマイグレーションファイルを複数の Repo に適用する必要があります。\nこれは、設定ファイルに以下の様に書くだけで出来ます。\n\nconfig/config.exs:\n\n```elixir\nconfig :yacto, :databases,\n  %{\n    default: %{\n      module: Yacto.DB.Single,\n      repo: MyApp.Repo.Default,\n    },\n    player: %{\n      module: Yacto.DB.Shard,\n      repos: [MyApp.Repo.Player0, MyApp.Repo.Player1],\n    },\n  }\n```\n\n`MyApp.Player` に以下のコードがあったことを思い出して下さい。\n\nlib/my_app/player.ex:\n\n```elixir\ndefmodule MyApp.Player do\n  use Yacto.Schema, dbname: :player\n\n  ...\n```\n\nこの `:player` が、`MyApp.Player` が所属する Repo のグループ名です。\n`MyApp.Player` は `:player` という Repo グループに所属しており、`:player` Repo グループは設定ファイルから `MyApp.Repo.Player0` と `MyApp.Repo.Player1` の Repo に紐付いていることが分かります。\n\n設定ファイルを書いたら、あとは `mix yacto.migrate` を実行するだけです。\n`MyApp.Player` のマイグレーションファイルが `MyApp.Repo.Player0` と `MyApp.Repo.Player1` に適用されます。\n\n水平分割したデータベースを利用する時には、`Yacto.DB.repo/2` を使って Repo を取得します。\n\n```elixir\nrepo = Yacto.DB.repo(:player, shard_key: player_id)\nMyApp.Player |> repo.all()\n```\n\n##","ref":"Yacto.html#module-%E6%B0%B4%E5%B9%B3%E5%88%86%E5%89%B2%E3%81%97%E3%81%9F%E3%83%87%E3%83%BC%E3%82%BF%E3%83%99%E3%83%BC%E3%82%B9%E3%81%B8%E3%81%AE%E3%83%9E%E3%82%A4%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3"},{"type":"module","title":"他のアプリケーションから利用する - Yacto","doc":"もちろん、この水平分割した `my_app` アプリケーションを `other_app` で利用することもできます。\n`other_app` で以下の様に設定ファイル書いて、\n\nconfig/config.exs:\n\n```elixir\nconfig :yacto, :databases,\n  %{\n    default: %{\n      module: Yacto.DB.Single,\n      repo: OtherApp.Repo.Default,\n    },\n    player: %{\n      module: Yacto.DB.Shard,\n      repos: [OtherApp.Repo.Player0, OtherApp.Repo.Player1, OtherApp.Repo.Player2],\n    },\n  }\n```\n\n`mix yacto.migrate --app my_app` を実行すると、`OtherApp.Repo.Player0` と `OtherApp.Repo.Player1` と `OtherApp.Repo.Player2` に `MyApp.Player` スキーマのマイグレーションファイルが適用されます。\n\n#","ref":"Yacto.html#module-%E4%BB%96%E3%81%AE%E3%82%A2%E3%83%97%E3%83%AA%E3%82%B1%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%81%8B%E3%82%89%E5%88%A9%E7%94%A8%E3%81%99%E3%82%8B"},{"type":"module","title":"複数データベースを跨るトランザクション（XA トランザクション） - Yacto","doc":"`Yacto.transaction/2` を使うと、複数のデータベースを指定してトランザクションを発行できます。\n\n```elixir\n# ２つ以上の Repo が指定されているので XA トランザクションを発行する\nYacto.transaction([:default,\n                   {:player, player_id1},\n                   {:player, player_id2}], fn ->\n  default_repo = Yacto.DB.repo(:default)\n  player1_repo = Yacto.DB.repo(:player, player_id1)\n  player2_repo = Yacto.DB.repo(:player, player_id2)\n\n  # ここら辺でデータベースを操作する\n  ...\n\n# ここで全ての XA トランザクションがコミットされる\nend)\n```\n\n以下の３つの Repo に対してトランザクションを行います。\n\n- `:default` の Repo `MyApp.Repo.Default`\n- `player_id1` でシャーディングされた Repo\n- `player_id2` でシャーディングされた Repo\n\n後ろの２つは、シャードキーによっては同じ Repo になる可能性があるので、利用する Repo は２つか３つのどちらかです。\n２つ以上の Repo を利用してトランザクションを開始する場合、自動的に XA トランザクションになります。\n\nXA トランザクションは確実に不整合が防げる訳ではありませんが、別々でトランザクションを発行するよりは防げます。\nただしこのライブラリでは `XA RECOVER` に残ったトランザクションを解決する仕組みを提供していないので、別途用意する必要があります。","ref":"Yacto.html#module-%E8%A4%87%E6%95%B0%E3%83%87%E3%83%BC%E3%82%BF%E3%83%99%E3%83%BC%E3%82%B9%E3%82%92%E8%B7%A8%E3%82%8B%E3%83%88%E3%83%A9%E3%83%B3%E3%82%B6%E3%82%AF%E3%82%B7%E3%83%A7%E3%83%B3-xa-%E3%83%88%E3%83%A9%E3%83%B3%E3%82%B6%E3%82%AF%E3%82%B7%E3%83%A7%E3%83%B3"},{"type":"module","title":"Yacto のスキーマ - Yacto","doc":"Yacto のスキーマについて、まだ説明していない部分があるので、もう少し詳しく説明します。\n\n最初に書いたように、Yacto のスキーマは以下の様に定義します。\n\n```elixir\ndefmodule MyApp.Player do\n  use Yacto.Schema, dbname: :player\n\n  schema @auto_source do\n    field :player_id, :string, meta: [null: false, size: 64]\n    field :hp, :integer, default: 0, meta: [null: false]\n    index :player_id, unique: true\n  end\nend\n```\n\n基本的には `Ecto.Schema` と変わりません。\n`Yacto.Schema` で生成したスキーマは、`Ecto.Schema` で生成したスキーマと互換性があります。\nただしマイグレーションに関する設定が含まれるので、`Ecto.Schema` よりもいくつか設定が増えています。\n\n#","ref":"Yacto.html#module-yacto-%E3%81%AE%E3%82%B9%E3%82%AD%E3%83%BC%E3%83%9E"},{"type":"module","title":"テーブル名の自動生成 - Yacto","doc":"`@auto_source` には、モジュール名から自動的に生成したテーブル名が定義されています。\n大抵の場合、自動的に決まった名前で問題ないと思うので、常に `@auto_source` を使うので問題ないでしょう。\n\n#","ref":"Yacto.html#module-%E3%83%86%E3%83%BC%E3%83%96%E3%83%AB%E5%90%8D%E3%81%AE%E8%87%AA%E5%8B%95%E7%94%9F%E6%88%90"},{"type":"module","title":"メタ情報 - Yacto","doc":"```elixir\nfield :player_id, :string, meta: [null: false, size: 64]\nfield :hp, :integer, default: 0, meta: [null: false]\nfield :height, :decimal, meta: [precision: 10, scale: 3]\n```\n\nここは `Ecto.Schema` の `field/3` 関数とほとんど同じですが、`:meta` オプションがあるという点で異なります。\n`:meta` オプションはマイグレーションに関する情報を入れる場所で、そのフィールドが null 可能かどうかや、文字列のサイズ等を指定します。\n\n指定可能なオプションは以下の通りです。\n\n- `:null`: そのフィールドが null 可能かどうか（デフォルトでは `true`）\n- `:size`: 文字列のサイズ（`VARCHAR(255)` の `255` に相当する部分）（デフォルトでは `255`）\n- `:default`: そのフィールドのデフォルト値（デフォルトでは各型の初期値か、`opts[:default]` が存在している場合はその値が入る）\n- `:precision`: `field/3` に渡す型が `:decimal` の場合の最大桁数を指定する（デフォルト値は利用する DB の仕様に従う）\n- `:scale`: `field/3` に渡す型が `:decimal` の場合の小数部の桁数を指定する（デフォルト値は利用する DB の仕様に従う）\n- `:index`: このフィールドでインデックスを張るかどうか（デフォルトでは `false`）\n- `:type`: マイグレーション時の型を指定する（デフォルトでは `field/3` で指定した型）\n\n#","ref":"Yacto.html#module-%E3%83%A1%E3%82%BF%E6%83%85%E5%A0%B1"},{"type":"module","title":"インデックス - Yacto","doc":"```elixir\nindex :player_id, unique: true\n```\n\n`index/2` でインデックスを生成できます。\n`field/3` の `:meta` オプションの中でもインデックスを指定できますが、`index/2` を使うと複合インデックスやユニークインデックスも生成できます。\n\n複合インデックスにするなら `index [:player_id, :hp]` のようにリストで指定します。\nユニークインデックスにするならオプションで `unique: true` を指定します。\n\n#","ref":"Yacto.html#module-%E3%82%A4%E3%83%B3%E3%83%87%E3%83%83%E3%82%AF%E3%82%B9"},{"type":"module","title":"外部キー制約 - Yacto","doc":"対応していません。\n必要だと思ったのであれば、ぜひ実装して pull req 下さい。","ref":"Yacto.html#module-%E5%A4%96%E9%83%A8%E3%82%AD%E3%83%BC%E5%88%B6%E7%B4%84"},{"type":"module","title":"便利関数 - Yacto","doc":"Yacto は、Repo に便利な関数を定義する `Yacto.Repo.Helper` を提供しています。\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n  use Yacto.Repo.Helper\nend\n```\n\nこれによって、以下の関数が定義されます。\n\n- `def count(queryable, clauses, opts \\\\ [])`\n\n`Ecto.Query.where` で絞り込んで要素数を返します。\n\n- `def find(queryable, clauses, opts \\\\ [])`\n\n`Ecto.Query.where` で絞り込んで Repo から取得します。\n\n- `def delete_by(queryable, clauses, opts \\\\ [])`\n- `def delete_by!(queryable, clauses, opts \\\\ [])`\n\n`Ecto.Query.where` で絞り込んで Repo から削除します。\n`delete_by!/3` は、削除した件数が 0 だった時に `Ecto.NoResultsError` 例外を投げます。\n\n- `def find_for_update(queryable, clauses, opts \\\\ [])`\n- `get_for_update(queryable, id, opts \\\\ [])`\n- `get_for_update!(queryable, id, opts \\\\ [])`\n- `get_by_for_update(queryable, clauses, opts \\\\ [])`\n- `get_by_for_update!(queryable, clauses, opts \\\\ [])`\n\n`SELECT ... FOR UPDATE` のクエリを使って要素を取得する関数です。\n\n`find_for_update/3` は `find/3` のクエリに `Ecto.Query.lock(\"FOR UPDATE\")` を付けただけの関数です。\n`get_for_update/3` や `get_by_for_update/3` 関数は、`Ecto.Repo.get` や `Ecto.Repo.get_by` のクエリに `Ecto.Query.lock(\"FOR UPDATE\")` を付けただけの関数です。\n\n\n- `get_by_or_new(queryable, clauses, default_struct, opts \\\\ [])`\n- `get_by_or_insert_for_update(queryable, clauses, default_struct_or_changeset, opts \\\\ [])`\n\n`get_by_or_new/4` は、まずレコードを取得してみて、あればそのレコードを、無ければデフォルト値 `default_struct` を返します。無かった場合でもデータベースへの挿入は行いません。\nロックを取らないので、この `get_by_or_new/4` で得られた値を使ってデータベースへ挿入や更新をしてはいけません。\n戻り値は `{record, defaulted}` の２要素のタプルになっていて、1要素目には取得できたレコード（あるいはデフォルト値）が、2要素目にはデフォルト値を返したかどうかのフラグが設定されます。\n\n`get_by_or_insert_for_update/4` は、`get_by_or_new/4` の排他ロックを取るバージョンです。\nまずレコードを取得してみて、あればそのレコードを、無ければ新規に `default_struct_or_changeset` を挿入して返します。この時、返されるレコードは排他ロックされます。\n戻り値は `{record, created}` の２要素のタプルになっていて、1要素目には取得できたレコード（あるいは挿入したデフォルト値）が、2要素目には新しく挿入したかどうかのフラグが設定されます。","ref":"Yacto.html#module-%E4%BE%BF%E5%88%A9%E9%96%A2%E6%95%B0"},{"type":"function","title":"Yacto.transaction/3","doc":"","ref":"Yacto.html#transaction/3"},{"type":"behaviour","title":"Yacto.DB","doc":"Yacto の repo を取得するためのモジュール\n\n水平分割をしている場合、シャーディングキーによって利用する repo が変わるため、\n直接 repo を利用するのではなく、`Yacto.repo/{2-3}` を利用する必要がある。","ref":"Yacto.DB.html"},{"type":"function","title":"Yacto.DB.all_repos/1","doc":"","ref":"Yacto.DB.html#all_repos/1"},{"type":"function","title":"Yacto.DB.repo/2","doc":"","ref":"Yacto.DB.html#repo/2"},{"type":"callback","title":"Yacto.DB.repo/3","doc":"","ref":"Yacto.DB.html#c:repo/3"},{"type":"function","title":"Yacto.DB.repos/2","doc":"","ref":"Yacto.DB.html#repos/2"},{"type":"callback","title":"Yacto.DB.repos/3","doc":"","ref":"Yacto.DB.html#c:repos/3"},{"type":"module","title":"Yacto.DB.Shard","doc":"","ref":"Yacto.DB.Shard.html"},{"type":"module","title":"Yacto.DB.Single","doc":"","ref":"Yacto.DB.Single.html"},{"type":"module","title":"Yacto.Migration.File","doc":"","ref":"Yacto.Migration.File.html"},{"type":"function","title":"Yacto.Migration.File.check_migration_files/1","doc":"","ref":"Yacto.Migration.File.html#check_migration_files/1"},{"type":"function","title":"Yacto.Migration.File.check_migrations/1","doc":"","ref":"Yacto.Migration.File.html#check_migrations/1"},{"type":"function","title":"Yacto.Migration.File.get_latest_migration_file/2","doc":"","ref":"Yacto.Migration.File.html#get_latest_migration_file/2"},{"type":"function","title":"Yacto.Migration.File.list_migration_files/2","doc":"","ref":"Yacto.Migration.File.html#list_migration_files/2"},{"type":"function","title":"Yacto.Migration.File.list_migration_modules/1","doc":"","ref":"Yacto.Migration.File.html#list_migration_modules/1"},{"type":"function","title":"Yacto.Migration.File.load_migration_module/2","doc":"","ref":"Yacto.Migration.File.html#load_migration_module/2"},{"type":"function","title":"Yacto.Migration.File.new/5","doc":"","ref":"Yacto.Migration.File.html#new/5"},{"type":"function","title":"Yacto.Migration.File.save/3","doc":"","ref":"Yacto.Migration.File.html#save/3"},{"type":"module","title":"Yacto.Migration.GenMigration","doc":"","ref":"Yacto.Migration.GenMigration.html"},{"type":"function","title":"Yacto.Migration.GenMigration.generate/3","doc":"","ref":"Yacto.Migration.GenMigration.html#generate/3"},{"type":"function","title":"Yacto.Migration.GenMigration.generate_fields/4","doc":"","ref":"Yacto.Migration.GenMigration.html#generate_fields/4"},{"type":"function","title":"Yacto.Migration.GenMigration.generate_indices/3","doc":"","ref":"Yacto.Migration.GenMigration.html#generate_indices/3"},{"type":"module","title":"Yacto.Migration.Migrator","doc":"","ref":"Yacto.Migration.Migrator.html"},{"type":"function","title":"Yacto.Migration.Migrator.migrate/6","doc":"","ref":"Yacto.Migration.Migrator.html#migrate/6"},{"type":"function","title":"Yacto.Migration.Migrator.migrated_versions/3","doc":"","ref":"Yacto.Migration.Migrator.html#migrated_versions/3"},{"type":"function","title":"Yacto.Migration.Migrator.up/6","doc":"","ref":"Yacto.Migration.Migrator.html#up/6"},{"type":"module","title":"Yacto.Migration.SchemaMigration","doc":"","ref":"Yacto.Migration.SchemaMigration.html"},{"type":"function","title":"Yacto.Migration.SchemaMigration.down/4","doc":"","ref":"Yacto.Migration.SchemaMigration.html#down/4"},{"type":"function","title":"Yacto.Migration.SchemaMigration.drop/1","doc":"","ref":"Yacto.Migration.SchemaMigration.html#drop/1"},{"type":"function","title":"Yacto.Migration.SchemaMigration.drop_and_create/1","doc":"","ref":"Yacto.Migration.SchemaMigration.html#drop_and_create/1"},{"type":"function","title":"Yacto.Migration.SchemaMigration.ensure_schema_migrations_table!/1","doc":"","ref":"Yacto.Migration.SchemaMigration.html#ensure_schema_migrations_table!/1"},{"type":"function","title":"Yacto.Migration.SchemaMigration.migrated_versions/3","doc":"","ref":"Yacto.Migration.SchemaMigration.html#migrated_versions/3"},{"type":"function","title":"Yacto.Migration.SchemaMigration.up/4","doc":"","ref":"Yacto.Migration.SchemaMigration.html#up/4"},{"type":"module","title":"Yacto.Migration.Structure","doc":"","ref":"Yacto.Migration.Structure.html"},{"type":"function","title":"Yacto.Migration.Structure.diff/2","doc":"","ref":"Yacto.Migration.Structure.html#diff/2"},{"type":"function","title":"Yacto.Migration.Structure.from_schema/1","doc":"","ref":"Yacto.Migration.Structure.html#from_schema/1"},{"type":"function","title":"Yacto.Migration.Structure.to_string/1","doc":"","ref":"Yacto.Migration.Structure.html#to_string/1"},{"type":"module","title":"Yacto.Migration.Util","doc":"","ref":"Yacto.Migration.Util.html"},{"type":"function","title":"Yacto.Migration.Util.allow_migrate?/3","doc":"","ref":"Yacto.Migration.Util.html#allow_migrate?/3"},{"type":"function","title":"Yacto.Migration.Util.get_all_schema/2","doc":"","ref":"Yacto.Migration.Util.html#get_all_schema/2"},{"type":"function","title":"Yacto.Migration.Util.get_migration_dir/1","doc":"","ref":"Yacto.Migration.Util.html#get_migration_dir/1"},{"type":"function","title":"Yacto.Migration.Util.get_migration_dir_for_gen/0","doc":"","ref":"Yacto.Migration.Util.html#get_migration_dir_for_gen/0"},{"type":"function","title":"Yacto.Migration.Util.is_migration_module?/1","doc":"","ref":"Yacto.Migration.Util.html#is_migration_module?/1"},{"type":"function","title":"Yacto.Migration.Util.is_schema_module?/1","doc":"","ref":"Yacto.Migration.Util.html#is_schema_module?/1"},{"type":"function","title":"Yacto.Migration.Util.need_gen_migration?/1","doc":"","ref":"Yacto.Migration.Util.html#need_gen_migration?/1"},{"type":"module","title":"Yacto.Query","doc":"クエリに関する便利関数を提供するモジュール","ref":"Yacto.Query.html"},{"type":"function","title":"Yacto.Query.for_update/1","doc":"`SELECT ... FOR UPDATE` のクエリを発行する","ref":"Yacto.Query.html#for_update/1"},{"type":"module","title":"Yacto.Repo.Helper","doc":"repo のためのヘルパ関数\n\n```\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n  use Yacto.Repo.Helper\nend\n\nperson = MyApp.Repo.get_or_insert_for_update(Person, [name: \"foo\"], %Person{name: \"foo\", value: 10})\n# これで person は排他ロックになる\n\nchangeset = Person.changeset(person, [value: person.value + 5])\n_person = MyApp.Repo.update!(changeset)\n```","ref":"Yacto.Repo.Helper.html"},{"type":"behaviour","title":"Yacto.Schema","doc":"Yacto 用のスキーマ\n\n以下のように利用する。\n\n```\ndefmodule MyApp.Schema.Item do\n  use Yacto.Schema, dbname: :default\n\n  schema @auto_source do\n    field :name, :string, meta: [null: false, size: 16, index: true]\n  end\nend\n\ndefmodule MyApp.Schema.Player do\n  use Yacto.Schema, dbname: :player\n\n  schema @auto_source do\n    field :name, :string, meta: [null: false, size: 16, index: true]\n  end\nend\n```","ref":"Yacto.Schema.html"},{"type":"callback","title":"Yacto.Schema.dbname/0","doc":"","ref":"Yacto.Schema.html#c:dbname/0"},{"type":"macro","title":"Yacto.Schema.field/3","doc":"","ref":"Yacto.Schema.html#field/3"},{"type":"macro","title":"Yacto.Schema.index/2","doc":"","ref":"Yacto.Schema.html#index/2"},{"type":"macro","title":"Yacto.Schema.schema/2","doc":"","ref":"Yacto.Schema.html#schema/2"},{"type":"macro","title":"Yacto.Schema.timestamps/1","doc":"","ref":"Yacto.Schema.html#timestamps/1"},{"type":"module","title":"Yacto.XA","doc":"XA トランザクションを行うためのモジュール","ref":"Yacto.XA.html"},{"type":"function","title":"Yacto.XA.in_xa_transaction?/1","doc":"","ref":"Yacto.XA.html#in_xa_transaction?/1"},{"type":"function","title":"Yacto.XA.rollback/2","doc":"","ref":"Yacto.XA.html#rollback/2"},{"type":"function","title":"Yacto.XA.transaction/3","doc":"","ref":"Yacto.XA.html#transaction/3"},{"type":"function","title":"Yacto.XA.with_xa/3","doc":"","ref":"Yacto.XA.html#with_xa/3"},{"type":"module","title":"Yacto.XA.RepoState","doc":"","ref":"Yacto.XA.RepoState.html"},{"type":"exception","title":"Yacto.XA.RollbackError","doc":"","ref":"Yacto.XA.RollbackError.html"},{"type":"extras","title":"変更履歴","doc":"# 変更履歴\n\n- UPDATE\n    - 下位互換がある変更\n- ADD\n    - 下位互換がある追加\n- CHANGE\n    - 下位互換のない変更\n- FIX\n    - バグ修正","ref":"changelog.html"},{"type":"extras","title":"develop - 変更履歴","doc":"","ref":"changelog.html#develop"},{"type":"extras","title":"2.0.5 (2023-12-23) - 変更履歴","doc":"- [UPDATE] Elixir 1.16 に対応\n- [UPDATE] 依存ライブラリを更新\n  - ecto_sql: 3.10.2 → 3.11.1\n  - memoize: 1.4.1 → 1.4.2\n  - ex_doc: 0.29.4 → 0.31.0\n  - myxql: 0.6.3 → 0.6.4\n- [UPDATE] Elixir 1.14 以上の場合、`Yacto.Migration.Structure` を inspect する際に Map データをソートして文字列に変換する\n- [CHANGE] サポートする Elixir のバージョンを 1.9 から 1.11 に上げる","ref":"changelog.html#2-0-5-2023-12-23"},{"type":"extras","title":"2.0.4 (2023-06-22) - 変更履歴","doc":"- [UPDATE] Elixir 1.15 に対応\n- [UPDATE] 依存ライブラリを更新\n  - ecto_sql: 3.8.3 → 3.10.2\n  - memoize: 1.4.1 → 1.4.2\n  - ex_doc: 0.28.5 → 0.29.4\n  - myxql: 0.6.2 → 0.6.3","ref":"changelog.html#2-0-4-2023-06-22"},{"type":"extras","title":"2.0.3 (2022-09-04) - 変更履歴","doc":"- [UPDATE] Elixir 1.14 に対応\n- [UPDATE] 依存ライブラリを更新\n  - ecto_sql: 3.7.2 → 3.8.3\n  - myxql: 0.6.1 → 0.6.2\n  - ex_doc: 0.28.2 → 0.28.5\n  - memoize: 1.4.0 → 1.4.1","ref":"changelog.html#2-0-3-2022-09-04"},{"type":"extras","title":"2.0.2 (2022-03-15) - 変更履歴","doc":"- [UPDATE] 依存ライブラリを更新\n  - ecto_sql: 3.6.0 → 3.7.2\n  - myxql: 0.5.1 → 0.6.1\n  - ex_doc: 0.25.0 → 0.28.2\n  - memoize: 1.3.3 → 1.4.0\n  - power_assert: 0.2.1 → 0.3.0","ref":"changelog.html#2-0-2-2022-03-15"},{"type":"extras","title":"2.0.1 (2021-04-04) - 変更履歴","doc":"- [UPDATE] 依存ライブラリを更新\n  - ecto_sql: 3.5.0 → 3.6.0\n  - myxql: 0.4.3 → 0.5.1\n  - ex_doc: 0.22.6 → 0.24.1\n  - memoize: 1.3.1 → 1.3.3\n  - power_assert: 0.2.0 → 0.2.1","ref":"changelog.html#2-0-1-2021-04-04"},{"type":"extras","title":"2.0.0 (2020-10-07) - 変更履歴","doc":"#","ref":"changelog.html#2-0-0-2020-10-07"},{"type":"extras","title":"下位互換のない変更 - 変更履歴","doc":"- [CHANGE] マイグレーションファイルをモデル毎に生成する\n  - 今まで生成した全てのマイグレーションファイルは利用できなくなります。\n  - 新しく生成したマイグレーションファイルで `mix yacto.migrate --fake` を実行して新しくマイグレーションスキーマを設定する必要があります。\n- [CHANGE] `Yacto.Schema` で定義していたデフォルトの `@primary_key` を削除した\n- [CHANGE] `@auto_source` から生成するテーブル名のルールを変更した\n  - 例えば `App.Model.FooBar` の場合、1.x では `app_model_foo_bar` になっていたが、2.x では `app_model_foobar` となる\n- [CHANGE] `Yacto.DB` の引数にキーワードリストを取れるようにして、データベースの設定を動的に渡せるようにした\n  - `Yacto.DB.repo(:player, \"key\")` と書いていたのを `Yacto.DB.repo(:player, shard_key: \"key\")` と書く必要がある\n- [CHANGE] `:table_name_converter` は不完全な機能だったので削除した\n- [CHANGE] `Yacto.Schema.Single` と `Yacto.Schema.Shard` を削除した\n  - データベースの設定を動的に渡せるようになり、`Yacto.Schema.Single` と `Yacto.Schema.Shard` のチェック機構が使えなくなるため。\n- [CHANGE] `mix yacto.migrate` で `--repo` を指定しなかった場合に利用するリポジトリ一覧を `:ecto_repos` から取得するようにした\n\n#","ref":"changelog.html#%E4%B8%8B%E4%BD%8D%E4%BA%92%E6%8F%9B%E3%81%AE%E3%81%AA%E3%81%84%E5%A4%89%E6%9B%B4"},{"type":"extras","title":"下位互換がある追加 - 変更履歴","doc":"- [ADD] スキーマの定義時にマイグレーションファイルを生成するかどうかを選択できるようにした（Thanks @mori5321 !）\n- [ADD] `field/3` のマイグレーション用メタ情報に `:precision` と `:scale` を追加\n- [ADD] モデルを削除した時に drop table する機能を実装\n- [ADD] マイグレーションファイルを検証する機能を実装\n  - 複数のブランチからマージした場合に矛盾が起きる可能性があったので、それを検出してエラーにする\n- [ADD] `Yacto.transaction/3` に、XA を行わない `:noxa` オプションを追加した\n\n#","ref":"changelog.html#%E4%B8%8B%E4%BD%8D%E4%BA%92%E6%8F%9B%E3%81%8C%E3%81%82%E3%82%8B%E8%BF%BD%E5%8A%A0"},{"type":"extras","title":"下位互換がある変更 - 変更履歴","doc":"- [UPDATE] Ecto 3.5 への対応\n  - MySQL Adapter をやめて MyXQL Adapter を使うようにした\n  - 新しいバージョンの Ecto.Migration.Runner に追従\n- [UPDATE] Elixir 1.11 への対応\n- [UPDATE] 全体的に日本語でやることにした\n\n#","ref":"changelog.html#%E4%B8%8B%E4%BD%8D%E4%BA%92%E6%8F%9B%E3%81%8C%E3%81%82%E3%82%8B%E5%A4%89%E6%9B%B4"},{"type":"extras","title":"バグ修正 - 変更履歴","doc":"- [FIX] インデックスの付いたフィールドを削除するとマイグレーションに失敗する問題を修正した (Thanks @h1u2i3 !)\n- [FIX] DB の新規作成時に、カラムの並びが定義順になるようにした\n- [FIX] `:index_name_max_length` 設定がある場合、長いインデックス名を削るようにした\n- [FIX] `get_for_update/3` を修正した\n- [FIX] テスト周りのリファクタリング","ref":"changelog.html#%E3%83%90%E3%82%B0%E4%BF%AE%E6%AD%A3"},{"type":"extras","title":"1.2.6 - 変更履歴","doc":"- Implement table name converter","ref":"changelog.html#1-2-6"},{"type":"extras","title":"1.2.5 - 変更履歴","doc":"- Use asdf\n- Fix migration error when using custom Ecto.Type with default value","ref":"changelog.html#1-2-5"},{"type":"extras","title":"1.2.4 - 変更履歴","doc":"- Customize a hashing function for sharding databases","ref":"changelog.html#1-2-4"},{"type":"extras","title":"1.2.3 - 変更履歴","doc":"- Fix a migration error is occured when using `:source` in field","ref":"changelog.html#1-2-3"},{"type":"extras","title":"1.2.2 - 変更履歴","doc":"- `timestamps/1` can be had `:meta`","ref":"changelog.html#1-2-2"},{"type":"extras","title":"1.2.1 - 変更履歴","doc":"- Fix an issue where using `autogenerate: {m, f, a}` in `@primary_key` definition causes an error\n- Implement `@primary_key_meta` that specify a primary key meta info for migration","ref":"changelog.html#1-2-1"},{"type":"extras","title":"1.2.0 - 変更履歴","doc":"- Implement `Yacto.XA.rollback/2`","ref":"changelog.html#1-2-0"},{"type":"extras","title":"1.1.3 - 変更履歴","doc":"- Update dependencies","ref":"changelog.html#1-1-3"},{"type":"extras","title":"1.1.2 - 変更履歴","doc":"- In `Yacto.Schema.field/3`, if opts[:default] is specified, set the value to meta[:default]","ref":"changelog.html#1-1-2"},{"type":"extras","title":"1.1.1 - 変更履歴","doc":"- Rename `get_or_new/4` to `get_by_or_new/4` and `get_or_insert_for_update/4` to `get_by_or_insert_for_update/4`","ref":"changelog.html#1-1-1"},{"type":"extras","title":"1.1.0 - 変更履歴","doc":"- Add `Yacto.Schema.Single` and `Yacto.Schema.Shard` modules.\n    - These define `repo/{0-1}` after `use Yacto.Schema`.\n- Add `Yacto.Repo.Helper` to provide convenient functions.","ref":"changelog.html#1-1-0"},{"type":"extras","title":"1.0.7 - 変更履歴","doc":"- Apply elixir formatter\n- Fix warnings\n- Update dependencies","ref":"changelog.html#1-0-7"},{"type":"extras","title":"1.0.6 - 変更履歴","doc":"- [Bug] Fix a migration type name includes with spaces.","ref":"changelog.html#1-0-6"},{"type":"extras","title":"1.0.5 - 変更履歴","doc":"- Shorten index name for too long name error.\n- Use specified migration type if :type defined in meta\n- Remove illegal character in filename for Windows","ref":"changelog.html#1-0-5"},{"type":"extras","title":"ライセンス","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation, and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"},{"type":"extras","title":"概要","doc":"# Yacto\n\n[![Module Version](https://img.shields.io/hexpm/v/yacto.svg)](https://hex.pm/packages/yacto)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/yacto/)\n[![Total Download](https://img.shields.io/hexpm/dt/yacto.svg)](https://hex.pm/packages/yacto)\n[![License](https://img.shields.io/hexpm/l/yacto.svg)](https://github.com/gumi/yacto/blob/master/LICENSE.md)\n[![Last Updated](https://img.shields.io/github/last-commit/gumi/yacto.svg)](https://github.com/gumi/yacto/commits/master)\n\n<!-- MDOC !-->\n\nYacto は、[Ecto](https://hexdocs.pm/ecto/Ecto.html) で手が届きにくい部分をサポートするためのライブラリです。\n\n大まかに以下の機能があります。\n\n- マイグレーションファイルの自動生成\n- 別アプリケーションからのマイグレーションの利用\n- 水平分割したデータベースへのマイグレーション\n- 複数データベースを跨るトランザクション（XA トランザクション）","ref":"readme.html"},{"type":"extras","title":"マイグレーションファイルの自動生成 - 概要","doc":"Yacto は、特にマイグレーション周りが Ecto と異なります。\nEcto はスキーマとマイグレーションを別で定義していましたが、Yacto は **スキーマからマイグレーションファイルを自動的に出力します**。\n\n具体的には、以下の様にスキーマを定義したとして、\n\nlib/my_app/player.ex:\n\n```elixir\ndefmodule MyApp.Player do\n  use Yacto.Schema, dbname: :player\n\n  schema @auto_source do\n    # sharding key\n    field :player_id, :string, meta: [null: false, size: 64]\n    field :hp, :integer, default: 0, meta: [null: false]\n    index :player_id, unique: true\n  end\nend\n```\n\nこの状態で `mix yacto.gen.migration` を実行すると、以下の様なマイグレーションファイルが出力されます。\n\n`priv/migrations/Elixir.MyApp.Player/0000-player-create-2020_04_22_085825.exs`:\n\n```elixir\ndefmodule MyApp.Player.Migration0000 do\n  use Ecto.Migration\n\n  def change() do\n    create table(\"myapp_player\")\n    alter table(\"myapp_player\") do\n      add(:hp, :integer, [null: false, size: 64])\n      add(:player_id, :string, [null: false])\n    end\n    create index(\"myapp_player\", [:player_id], [name: \"player_id_index\", unique: true])\n  end\n\n  def __migration__(:structure) do\n    {MyApp.Player, %Yacto.Migration.Structure{fields: [:id, :player_id, :hp], meta: %{attrs: %{hp: %{null: false}, player_id: %{null: false, size: 64}}, indices: %{{[:player_id], [unique: true]} => true}}, source: \"myapp_player\", types: %{hp: :integer, id: :id, player_id: :string}}},\n  end\n\n  def __migration__(:version) do\n    0\n  end\nend\n```\n\nあとは `mix yacto.migrate` を実行すれば、このマイグレーションファイルがデータベースに反映されます。\nもうマイグレーションファイルを自分で記述する必要はありません。\n\n更に、この状態で `MyApp.Player` スキーマに `:mp` フィールドを追加して `mix yacto.gen.migration` を実行すると、以下のマイグレーションファイルが生成されます。\n\nlib/my_app/player.ex:\n\n```elixir\ndefmodule MyApp.Player do\n  ...\n\n  schema @auto_source do\n    ...\n\n    # add a field\n    field :mp, :integer, default: 0, meta: [null: false]\n\n    ...\n  end\nend\n```\n\n`priv/migrations/Elixir.MyApp.Player/0001-player-change-2020_04_22_092258.exs`:\n\n```elixir\ndefmodule MyApp.Player.Migration0001 do\n  use Ecto.Migration\n\n  def change() do\n    alter table(\"myapp_player\") do\n      add(:mp, :integer, [null: false])\n    end\n  end\n\n  def __migration__(:structure) do\n    [\n      {MyApp.Player, %Yacto.Migration.Structure{fields: [:id, :player_id, :hp, :mp], meta: %{attrs: %{hp: %{null: false}, mp: %{null: false}, player_id: %{null: false, size: 64}}, indices: %{{[:player_id], [unique: true]} => true}}, source: \"myapp_player\", types: %{hp: :integer, id: :id, mp: :integer, player_id: :string}}},\n    ]\n  end\n\n  def __migration__(:version) do\n    1\n  end\nend\n```\n\nこのように、以前からの差分だけがマイグレーションファイルに出力されます。\n`mix yacto.migrate` を実行すれば、このマイグレーションファイルがデータベースに反映されます。\n\nもしマイグレーションファイルが１つもデータベースに適用されていなかったら、上記の２つのマイグレーションファイルが順番に適用されます。\n\n#","ref":"readme.html#%E3%83%9E%E3%82%A4%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%83%95%E3%82%A1%E3%82%A4%E3%83%AB%E3%81%AE%E8%87%AA%E5%8B%95%E7%94%9F%E6%88%90"},{"type":"extras","title":"別アプリケーションからのマイグレーションの利用 - 概要","doc":"先程作った `my_app` アプリケーションを利用する `other_app` アプリケーションがあったとします。\n`my_app` はデータベース利用しているので、`other_app` 上で `my_app` のためのマイグレーションを行う必要があります。\nYacto を使えば、`config/config.exs` を適切に書いた後、`other_app` で以下のコマンドを実行するだけで `my_app` のマイグレーションができます。\n\n```\nmix yacto.migrate --app my_app\n```\n\nEcto では、他のアプリケーションが必要としているマイグレーションを自分で書くか、各アプリケーションが指定したバラバラな方法でマイグレーションを行う必要がありました。\nYacto を使っているアプリケーションでは、全て同じ方法でマイグレーションができます。\n\n#","ref":"readme.html#%E5%88%A5%E3%82%A2%E3%83%97%E3%83%AA%E3%82%B1%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%81%8B%E3%82%89%E3%81%AE%E3%83%9E%E3%82%A4%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%81%AE%E5%88%A9%E7%94%A8"},{"type":"extras","title":"水平分割したデータベースへのマイグレーション - 概要","doc":"例えば `MyApp.Player` スキーマを水平分割した場合、このスキーマのマイグレーションファイルを複数の Repo に適用する必要があります。\nこれは、設定ファイルに以下の様に書くだけで出来ます。\n\nconfig/config.exs:\n\n```elixir\nconfig :yacto, :databases,\n  %{\n    default: %{\n      module: Yacto.DB.Single,\n      repo: MyApp.Repo.Default,\n    },\n    player: %{\n      module: Yacto.DB.Shard,\n      repos: [MyApp.Repo.Player0, MyApp.Repo.Player1],\n    },\n  }\n```\n\n`MyApp.Player` に以下のコードがあったことを思い出して下さい。\n\nlib/my_app/player.ex:\n\n```elixir\ndefmodule MyApp.Player do\n  use Yacto.Schema, dbname: :player\n\n  ...\n```\n\nこの `:player` が、`MyApp.Player` が所属する Repo のグループ名です。\n`MyApp.Player` は `:player` という Repo グループに所属しており、`:player` Repo グループは設定ファイルから `MyApp.Repo.Player0` と `MyApp.Repo.Player1` の Repo に紐付いていることが分かります。\n\n設定ファイルを書いたら、あとは `mix yacto.migrate` を実行するだけです。\n`MyApp.Player` のマイグレーションファイルが `MyApp.Repo.Player0` と `MyApp.Repo.Player1` に適用されます。\n\n水平分割したデータベースを利用する時には、`Yacto.DB.repo/2` を使って Repo を取得します。\n\n```elixir\nrepo = Yacto.DB.repo(:player, shard_key: player_id)\nMyApp.Player |> repo.all()\n```\n\n##","ref":"readme.html#%E6%B0%B4%E5%B9%B3%E5%88%86%E5%89%B2%E3%81%97%E3%81%9F%E3%83%87%E3%83%BC%E3%82%BF%E3%83%99%E3%83%BC%E3%82%B9%E3%81%B8%E3%81%AE%E3%83%9E%E3%82%A4%E3%82%B0%E3%83%AC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3"},{"type":"extras","title":"他のアプリケーションから利用する - 概要","doc":"もちろん、この水平分割した `my_app` アプリケーションを `other_app` で利用することもできます。\n`other_app` で以下の様に設定ファイル書いて、\n\nconfig/config.exs:\n\n```elixir\nconfig :yacto, :databases,\n  %{\n    default: %{\n      module: Yacto.DB.Single,\n      repo: OtherApp.Repo.Default,\n    },\n    player: %{\n      module: Yacto.DB.Shard,\n      repos: [OtherApp.Repo.Player0, OtherApp.Repo.Player1, OtherApp.Repo.Player2],\n    },\n  }\n```\n\n`mix yacto.migrate --app my_app` を実行すると、`OtherApp.Repo.Player0` と `OtherApp.Repo.Player1` と `OtherApp.Repo.Player2` に `MyApp.Player` スキーマのマイグレーションファイルが適用されます。\n\n#","ref":"readme.html#%E4%BB%96%E3%81%AE%E3%82%A2%E3%83%97%E3%83%AA%E3%82%B1%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E3%81%8B%E3%82%89%E5%88%A9%E7%94%A8%E3%81%99%E3%82%8B"},{"type":"extras","title":"複数データベースを跨るトランザクション（XA トランザクション） - 概要","doc":"`Yacto.transaction/2` を使うと、複数のデータベースを指定してトランザクションを発行できます。\n\n```elixir\n# ２つ以上の Repo が指定されているので XA トランザクションを発行する\nYacto.transaction([:default,\n                   {:player, player_id1},\n                   {:player, player_id2}], fn ->\n  default_repo = Yacto.DB.repo(:default)\n  player1_repo = Yacto.DB.repo(:player, player_id1)\n  player2_repo = Yacto.DB.repo(:player, player_id2)\n\n  # ここら辺でデータベースを操作する\n  ...\n\n# ここで全ての XA トランザクションがコミットされる\nend)\n```\n\n以下の３つの Repo に対してトランザクションを行います。\n\n- `:default` の Repo `MyApp.Repo.Default`\n- `player_id1` でシャーディングされた Repo\n- `player_id2` でシャーディングされた Repo\n\n後ろの２つは、シャードキーによっては同じ Repo になる可能性があるので、利用する Repo は２つか３つのどちらかです。\n２つ以上の Repo を利用してトランザクションを開始する場合、自動的に XA トランザクションになります。\n\nXA トランザクションは確実に不整合が防げる訳ではありませんが、別々でトランザクションを発行するよりは防げます。\nただしこのライブラリでは `XA RECOVER` に残ったトランザクションを解決する仕組みを提供していないので、別途用意する必要があります。","ref":"readme.html#%E8%A4%87%E6%95%B0%E3%83%87%E3%83%BC%E3%82%BF%E3%83%99%E3%83%BC%E3%82%B9%E3%82%92%E8%B7%A8%E3%82%8B%E3%83%88%E3%83%A9%E3%83%B3%E3%82%B6%E3%82%AF%E3%82%B7%E3%83%A7%E3%83%B3-xa-%E3%83%88%E3%83%A9%E3%83%B3%E3%82%B6%E3%82%AF%E3%82%B7%E3%83%A7%E3%83%B3"},{"type":"extras","title":"Yacto のスキーマ - 概要","doc":"Yacto のスキーマについて、まだ説明していない部分があるので、もう少し詳しく説明します。\n\n最初に書いたように、Yacto のスキーマは以下の様に定義します。\n\n```elixir\ndefmodule MyApp.Player do\n  use Yacto.Schema, dbname: :player\n\n  schema @auto_source do\n    field :player_id, :string, meta: [null: false, size: 64]\n    field :hp, :integer, default: 0, meta: [null: false]\n    index :player_id, unique: true\n  end\nend\n```\n\n基本的には `Ecto.Schema` と変わりません。\n`Yacto.Schema` で生成したスキーマは、`Ecto.Schema` で生成したスキーマと互換性があります。\nただしマイグレーションに関する設定が含まれるので、`Ecto.Schema` よりもいくつか設定が増えています。\n\n#","ref":"readme.html#yacto-%E3%81%AE%E3%82%B9%E3%82%AD%E3%83%BC%E3%83%9E"},{"type":"extras","title":"テーブル名の自動生成 - 概要","doc":"`@auto_source` には、モジュール名から自動的に生成したテーブル名が定義されています。\n大抵の場合、自動的に決まった名前で問題ないと思うので、常に `@auto_source` を使うので問題ないでしょう。\n\n#","ref":"readme.html#%E3%83%86%E3%83%BC%E3%83%96%E3%83%AB%E5%90%8D%E3%81%AE%E8%87%AA%E5%8B%95%E7%94%9F%E6%88%90"},{"type":"extras","title":"メタ情報 - 概要","doc":"```elixir\nfield :player_id, :string, meta: [null: false, size: 64]\nfield :hp, :integer, default: 0, meta: [null: false]\nfield :height, :decimal, meta: [precision: 10, scale: 3]\n```\n\nここは `Ecto.Schema` の `field/3` 関数とほとんど同じですが、`:meta` オプションがあるという点で異なります。\n`:meta` オプションはマイグレーションに関する情報を入れる場所で、そのフィールドが null 可能かどうかや、文字列のサイズ等を指定します。\n\n指定可能なオプションは以下の通りです。\n\n- `:null`: そのフィールドが null 可能かどうか（デフォルトでは `true`）\n- `:size`: 文字列のサイズ（`VARCHAR(255)` の `255` に相当する部分）（デフォルトでは `255`）\n- `:default`: そのフィールドのデフォルト値（デフォルトでは各型の初期値か、`opts[:default]` が存在している場合はその値が入る）\n- `:precision`: `field/3` に渡す型が `:decimal` の場合の最大桁数を指定する（デフォルト値は利用する DB の仕様に従う）\n- `:scale`: `field/3` に渡す型が `:decimal` の場合の小数部の桁数を指定する（デフォルト値は利用する DB の仕様に従う）\n- `:index`: このフィールドでインデックスを張るかどうか（デフォルトでは `false`）\n- `:type`: マイグレーション時の型を指定する（デフォルトでは `field/3` で指定した型）\n\n#","ref":"readme.html#%E3%83%A1%E3%82%BF%E6%83%85%E5%A0%B1"},{"type":"extras","title":"インデックス - 概要","doc":"```elixir\nindex :player_id, unique: true\n```\n\n`index/2` でインデックスを生成できます。\n`field/3` の `:meta` オプションの中でもインデックスを指定できますが、`index/2` を使うと複合インデックスやユニークインデックスも生成できます。\n\n複合インデックスにするなら `index [:player_id, :hp]` のようにリストで指定します。\nユニークインデックスにするならオプションで `unique: true` を指定します。\n\n#","ref":"readme.html#%E3%82%A4%E3%83%B3%E3%83%87%E3%83%83%E3%82%AF%E3%82%B9"},{"type":"extras","title":"外部キー制約 - 概要","doc":"対応していません。\n必要だと思ったのであれば、ぜひ実装して pull req 下さい。","ref":"readme.html#%E5%A4%96%E9%83%A8%E3%82%AD%E3%83%BC%E5%88%B6%E7%B4%84"},{"type":"extras","title":"便利関数 - 概要","doc":"Yacto は、Repo に便利な関数を定義する `Yacto.Repo.Helper` を提供しています。\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n  use Yacto.Repo.Helper\nend\n```\n\nこれによって、以下の関数が定義されます。\n\n- `def count(queryable, clauses, opts \\\\ [])`\n\n`Ecto.Query.where` で絞り込んで要素数を返します。\n\n- `def find(queryable, clauses, opts \\\\ [])`\n\n`Ecto.Query.where` で絞り込んで Repo から取得します。\n\n- `def delete_by(queryable, clauses, opts \\\\ [])`\n- `def delete_by!(queryable, clauses, opts \\\\ [])`\n\n`Ecto.Query.where` で絞り込んで Repo から削除します。\n`delete_by!/3` は、削除した件数が 0 だった時に `Ecto.NoResultsError` 例外を投げます。\n\n- `def find_for_update(queryable, clauses, opts \\\\ [])`\n- `get_for_update(queryable, id, opts \\\\ [])`\n- `get_for_update!(queryable, id, opts \\\\ [])`\n- `get_by_for_update(queryable, clauses, opts \\\\ [])`\n- `get_by_for_update!(queryable, clauses, opts \\\\ [])`\n\n`SELECT ... FOR UPDATE` のクエリを使って要素を取得する関数です。\n\n`find_for_update/3` は `find/3` のクエリに `Ecto.Query.lock(\"FOR UPDATE\")` を付けただけの関数です。\n`get_for_update/3` や `get_by_for_update/3` 関数は、`Ecto.Repo.get` や `Ecto.Repo.get_by` のクエリに `Ecto.Query.lock(\"FOR UPDATE\")` を付けただけの関数です。\n\n\n- `get_by_or_new(queryable, clauses, default_struct, opts \\\\ [])`\n- `get_by_or_insert_for_update(queryable, clauses, default_struct_or_changeset, opts \\\\ [])`\n\n`get_by_or_new/4` は、まずレコードを取得してみて、あればそのレコードを、無ければデフォルト値 `default_struct` を返します。無かった場合でもデータベースへの挿入は行いません。\nロックを取らないので、この `get_by_or_new/4` で得られた値を使ってデータベースへ挿入や更新をしてはいけません。\n戻り値は `{record, defaulted}` の２要素のタプルになっていて、1要素目には取得できたレコード（あるいはデフォルト値）が、2要素目にはデフォルト値を返したかどうかのフラグが設定されます。\n\n`get_by_or_insert_for_update/4` は、`get_by_or_new/4` の排他ロックを取るバージョンです。\nまずレコードを取得してみて、あればそのレコードを、無ければ新規に `default_struct_or_changeset` を挿入して返します。この時、返されるレコードは排他ロックされます。\n戻り値は `{record, created}` の２要素のタプルになっていて、1要素目には取得できたレコード（あるいは挿入したデフォルト値）が、2要素目には新しく挿入したかどうかのフラグが設定されます。\n\n<!-- MDOC !-->","ref":"readme.html#%E4%BE%BF%E5%88%A9%E9%96%A2%E6%95%B0"},{"type":"extras","title":"Copyright and License - 概要","doc":"Apache License 2.0\n\n```\nCopyright 2017 gumi Inc.\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n```","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown"}